import os
from unittest import TestCase
from unittest.async_case import IsolatedAsyncioTestCase

from aiohttp import ClientSession
from polygonscan.core.async_client import AsyncClient
from polygonscan.core.sync_client import SyncClient

from requests import Session

CONFIG_PATH = "polygon/configs/stable.json"
API_KEY = os.environ["API_KEY"]


class TestSyncSession(TestCase):
    def test_from_session_sync(self):
        client = SyncClient.from_session(
            api_key=API_KEY,
            session=Session(),
        )
        self.assertTrue(client.get_matic_last_price())


class TestAsyncSession(IsolatedAsyncioTestCase):
    async def test_from_session_async(self):
        client = await AsyncClient.from_session(
            api_key=API_KEY,
            session=ClientSession(),
        )
        self.assertTrue(await client.get_matic_last_price())


import requests


class ResponseParser:
    @staticmethod
    def parse(response: dict):
        result = response["result"]
        if "status" in response.keys():
            status = bool(int(response["status"]))
            message = response["message"]
            assert status, f"{result} -- {message}"
        else:
            # GETH or Parity proxy msg format
            # TODO: see if we need those values
            jsonrpc = response["jsonrpc"]
            cid = int(response["id"])
        return result


import polygonscan
from aiohttp import ClientSession
from polygonscan.core.base import BaseClient
from polygonscan.enums.fields_enum import FieldsEnum as fields
from polygonscan.utils.parsing import ResponseParser as parser


class AsyncClient(BaseClient):
    async def _build(self):
        for func, v in self._config.items():
            if not func.startswith("_"):  # disabled if _
                attr = getattr(getattr(polygonscan, v["module"]), func)
                setattr(self, func, await self._exec(attr))
        return self

    async def _exec(self, func):
        async def wrapper(*args, **kwargs):
            url = (
                f"{fields.PREFIX}"
                f"{func(*args, **kwargs)}"
                f"{fields.API_KEY}"
                f"{self._api_key}"
            )
            if self._debug:
                print(f"\n{url}\n")
            async with self._session.get(url) as response:
                return parser.parse(await response.json())

        return wrapper

    async def __aenter__(self):
        self._session = ClientSession()
        return await self._build()

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self._session.close()

    @classmethod
    async def from_session(cls, api_key: str, session: ClientSession, **kwargs):
        client = AsyncClient(api_key, **kwargs)
        client._session = session
        return await client._build()


import json
from importlib import resources

from polygonscan.core.async_client import AsyncClient
from polygonscan.core.base import BaseClient
from polygonscan.core.sync_client import SyncClient


class PolygonScan:
    """Client factory."""

    def __new__(cls, api_key: str, asynchronous=True, debug=False) -> BaseClient:
        """Create a new client.
        Args:
            api_key (str): Your polygonscan.com API key.
            asynchronous (bool, optional): Whether client is async or not. Defaults to True.
            debug (bool, optional): Display generated URLs for debugging. Defaults to False.
        Returns:
            BaseClient: polygonscan client.
        """
        if asynchronous:
            return AsyncClient(api_key=api_key, debug=debug)
        return SyncClient(api_key=api_key, debug=debug)


from polygonscan.enums.actions_enum import ActionsEnum as actions
from polygonscan.enums.fields_enum import FieldsEnum as fields
from polygonscan.enums.modules_enum import ModulesEnum as modules


class Logs:
    @staticmethod
    def get_logs(
        from_block: int,
        to_block: int,
        address: str,
        topic_0: str = "",
        topic_1: str = "",
        topic_2: str = "",
        topic_3: str = "",
        topic_0_1_opr: str = "",
        topic_1_2_opr: str = "",
        topic_2_3_opr: str = "",
        topic_0_2_opr: str = "",
        topic_0_3_opr: str = "",
        topic_1_3_opr: str = "",
    ):
        """This is an alternative to the native eth_getLogs. An address and/or
        topic_x parameters are required. When multiple topic_x parameters are
        used, the topic_x_y_opr ("and"/"or" operator) is also required.
        **NOTE: Only the first 1000 results are returned.**
        Args:
            from_block (int): Start block of the query.
            to_block (int): End block of the query.
            address (str): Address of the logs.
            topic_0 (str, optional): Topic 0 in the logs. Defaults to "".
            topic_1 (str, optional): Topic 1 in the logs. Defaults to "".
            topic_2 (str, optional): Topic 2 in the logs. Defaults to "".
            topic_3 (str, optional): Topic 3 in the logs. Defaults to "".
            topic_0_1_opr (str, optional): Logical operator between topic 0 and 1. Defaults to "".
            topic_1_2_opr (str, optional): Logical operator between topic 1 and 2. Defaults to "".
            topic_2_3_opr (str, optional): Logical operator between topic 2 and 3. Defaults to "".
            topic_0_2_opr (str, optional): Logical operator between topic 0 and 2. Defaults to "".
            topic_0_3_opr (str, optional): Logical operator between topic 0 and 3. Defaults to "".
            topic_1_3_opr (str, optional): Logical operator between topic 1 and 3. Defaults to "".
        Returns:
            dict: The event logs in a dictionary, including topics and data fields.
        Example::
            from polygonscan import PolygonScan
            async with PolygonScan(YOUR_API_KEY) as client:
                print(
                    await client.get_logs(
                        from_block=4993830,
                        to_block=4993832,
                        address="0xe561479bebee0e606c19bb1973fc4761613e3c42",
                        topic_0="0xddf252ad1be2c89b69c2b068fc378daa952ba7f163c4a11628f55a4df523b3ef",
                        topic_0_1_opr="and",
                        topic_1="0x000000000000000000000000730e2065b9daee84c3003c05bf6d2b3a08e55667"
                    )
                )
        Results::
            [
                {
                "address": "0xe561479bebee0e606c19bb1973fc4761613e3c42",
                "topics": [
                    "0xddf252ad1be2c89b69c2b068fc378daa952ba7f163c4a11628f55a4df523b3ef",
                    "0x000000000000000000000000730e2065b9daee84c3003c05bf6d2b3a08e55667",
                    "0x000000000000000000000000d7d19938eae260d7f0e0a4c36e665ff4cf4b7acc"
                ],
                "data": "0x000000000000000000000000000000000000000000000000076cd96f53f24b0a",
                "blockNumber": "0x4c3326",
                "timeStamp": "0x602e9ef1",
                "gasPrice": "0x2540be400",
                "gasUsed": "0x1b0f2",
                "logIndex": "0xf7",
                "transactionHash": "0x73844fcfc6beab2e973a897c9573f4d79811b12213ce263045a203e0d3cea90e",
                "transactionIndex": "0xb9"
                }
            ]
        """
        return (
            f"{fields.MODULE}"
            f"{modules.LOGS}"
            f"{fields.ACTION}"
            f"{actions.GET_LOGS}"
            f"{fields.FROM_BLOCK}"
            f"{from_block}"
            f"{fields.TO_BLOCK}"
            f"{to_block}"
            f"{fields.ADDRESS}"
            f"{address}"
            # topic 0
            f"{fields.TOPIC_0}"
            f"{topic_0}"
            #
            # Everything below is optional. If not provided by user, then
            # they remain empty and do not affect the tail of the url.
            #
            # topic 0_x operators
            f"{fields.TOPIC_0_1_OPR*bool(topic_0_1_opr)}"
            f"{topic_0_1_opr}"
            f"{fields.TOPIC_0_2_OPR*bool(topic_0_2_opr)}"
            f"{topic_0_2_opr}"
            f"{fields.TOPIC_0_3_OPR*bool(topic_0_3_opr)}"
            f"{topic_0_3_opr}"
            # topic 1
            f"{fields.TOPIC_1*bool(topic_1)}"
            f"{topic_1}"
            # topic 1_x operators
            f"{fields.TOPIC_1_2_OPR*bool(topic_1_2_opr)}"
            f"{topic_1_2_opr}"
            f"{fields.TOPIC_1_3_OPR*bool(topic_1_3_opr)}"
            f"{topic_1_3_opr}"
            # topic 2
            f"{fields.TOPIC_2*bool(topic_2)}"
            f"{topic_2}"
            # topic 2_x operators
            f"{fields.TOPIC_2_3_OPR*bool(topic_2_3_opr)}"
            f"{topic_2_3_opr}"
            # topic 3
            f"{fields.TOPIC_3*bool(topic_3)}"
            f"{topic_3}"
        )


import polygonscan
from polygonscan.core.base import BaseClient
from polygonscan.enums.fields_enum import FieldsEnum as fields
from polygonscan.utils.parsing import ResponseParser as parser
from requests import Session


class SyncClient(BaseClient):
    def _build(self):
        for func, v in self._config.items():
            if not func.startswith("_"):  # disabled if _
                attr = getattr(getattr(polygonscan, v["module"]), func)
                setattr(self, func, self._exec(attr))
        return self

    def _exec(self, func):
        def wrapper(*args, **kwargs):
            url = (
                f"{fields.PREFIX}"
                f"{func(*args, **kwargs)}"
                f"{fields.API_KEY}"
                f"{self._api_key}"
            )
            if self._debug:
                print(f"\n{url}\n")
            with self._session.get(url) as response:
                return parser.parse(response.json())

        return wrapper

    def __enter__(self):
        self._session = Session()
        return self._build()

    def __exit__(self, exc_type, exc_val, exc_tb):
        self._session.close()

    @classmethod
    def from_session(cls, api_key: str, session: Session, **kwargs):
        client = SyncClient(api_key, **kwargs)
        client._session = session
        return client._build()


import json
from importlib import resources

from polygonscan import configs

CONFIG_FILE = "stable.json"


class BaseClient:
    def __init__(
        self,
        api_key: str,
        debug: bool = False,  # display generated URLs for debugging purposes
    ):
        self._config = self._load_config()
        self._api_key = api_key
        self._debug = debug

    @staticmethod
    def _load_config(config_file: str = CONFIG_FILE) -> dict:
        with resources.path(configs, config_file) as path, open(path, "r") as f:
            return json.load(f)


from setuptools import setup

setup(
    name="polygonscan-python",
    version="1.0.2",
    description="A minimal, complete, python API for polygonscan.com.",
    url="https://github.com/tarsil/polygonscan-python",
    author="Tiago A. Silva",
    license="MIT",
    packages=[
        "polygonscan",
        "polygonscan.configs",
        "polygonscan.core",
        "polygonscan.enums",
        "polygonscan.modules",
        "polygonscan.utils",
    ],
    python_requires='>=3.8',
    install_requires=["aiohttp", "requests"],
    include_package_data=True,
    zip_safe=False,
)



from polygonscan.enums.actions_enum import ActionsEnum as actions
from polygonscan.enums.fields_enum import FieldsEnum as fields
from polygonscan.enums.modules_enum import ModulesEnum as modules


class Stats:
    @staticmethod
    def get_total_matic_supply() -> str:
        url = (
            f"{fields.MODULE}"
            f"{modules.STATS}"
            f"{fields.ACTION}"
            f"{actions.ETH_SUPPLY}"
        )
        return url

    @staticmethod
    def get_matic_last_price() -> str:
        url = (
            f"{fields.MODULE}"
            f"{modules.STATS}"
            f"{fields.ACTION}"
            f"{actions.ETH_PRICE}"
        )
        return url
